from sklearn.feature_extraction.text import CountVectorizer
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.feature_extraction.text import HashingVectorizer
from sklearn.feature_extraction.text import TfidfTransformer
import re
import sys
from sklearn.cluster import KMeans, MiniBatchKMeans
from sklearn.decomposition import TruncatedSVD
from sklearn.preprocessing import Normalizer
from sklearn import metrics
from sklearn.pipeline import make_pipeline
from nltk.tag import pos_tag
from nltk.stem.snowball import SnowballStemmer
import nltk
import operator
from sklearn import svm
from sklearn.model_selection import cross_val_score

from nltk.tag.stanford import StanfordNERTagger
st = StanfordNERTagger('english.all.3class.distsim.crf.ser.gz','stanford-ner.jar')

#fs = sys.argv[1]

#use word stems
stemmer = SnowballStemmer("english")

##### FUNCTIONS #####
def tokenize_and_stem(text):
    # first tokenize by sentence, then by word to ensure that punctuation is caught as it's own token
    tokens = [word for sent in nltk.sent_tokenize(text) for word in nltk.word_tokenize(sent)]
    filtered_tokens = []
    # filter out any tokens not containing letters (e.g., numeric tokens, raw punctuation)
    for token in tokens:
            if re.search('[a-zA-Z]', token):
                    filtered_tokens.append(token)
    stems = [stemmer.stem(t) for t in filtered_tokens]
    return stems

def tokenize_only(text):
    # first tokenize by sentence, then by word to ensure that punctuation is caught as it's own token
        tokens = [word.lower() for sent in nltk.sent_tokenize(text) for word in nltk.word_tokenize(sent)]
        filtered_tokens = []
    # filter out any tokens not containing letters (e.g., numeric tokens, raw punctuation)
        for token in tokens:
            if re.search('[a-zA-Z]', token):
                    filtered_tokens.append(token)
        return filtered_tokens

#use regular expression to split by space
REGEX = re.compile(r"\s+")
def tokenize(text):
    return [tok.strip().lower() for tok in REGEX.split(text)]

##### MAIN #####

#open file and read data
f = open("spam_train.csv")
#f = open("input.csv")
#f = open("mydata.csv")
#tags=f.readlines()
data = []
labels=[]
line=f.readline()
line=f.readline()
l2 = ""
toremove = {}
while(line!=''):
        #remove non-ascii from the line
        line = re.sub(r"[^\x00-\x7F]+", "", line)
        l=line.split(',',1)
        if(l[0] == 'ham'):
                labels.append(1)
        else:
                labels.append(0)
        data.append(l[1])
        l2 += " " + l[1]
        line=f.readline()

'''
#tag words with the stanford ner tagger
l = nltk.sent_tokenize(l2)
tokens = nltk.tokenize.word_tokenize(l[0])
st_tags = st.tag(tokens)
for st_tag in st_tags:
        if st_tag[1]=='PERSON' or st_tag[1] == 'LOCATION':
                toremove[st_tag[0]]=1

##Remove words that appear in the toremove dictionary
for i in range(0, len(data), 1):
    l = []
    l.append(data[i])
    tokens = nltk.tokenize.word_tokenize(l[0])
    newstr = ""
    for tok in tokens:
                if(toremove.get(tok) == None):
                        newstr += tok+" "
    data[i] = newstr.lower()

for i in range(0, len(data), 1):
	tagged = pos_tag(data[i].split())
	toremove = [word for word,pos in tagged if( pos == 'CC' or pos == 'CD' or pos == 'DT' or pos == 'EX' or \
							pos == 'IN' or pos == 'LS' or pos == 'MD' or pos == 'DT' or \
						    	pos == 'PRP' or pos == 'PRP$' or pos == 'RP' or pos == 'TO' or \
							pos == 'UH' or pos == 'WDT' or pos == 'WP' or pos == 'WP$' ) ]
#							pos == 'JJ' or pos == 'JJR' or pos == 'JJS' or pos == 'PDT' or \
#							pos == 'VB' or pos == 'VBD' or pos == 'VBG' or pos == 'VBN' or \
#							pos == 'VBP' ) ]

#	print(data[i], " TAGS: ", toremove)
	tok = tokenize_only(data[i])
#	print(tok)
	newstr = ""
	for t in tok:
		flag=0
		for j in range(0, len(toremove), 1):
			if(toremove[j] == t):
				flag=1
		if(flag == 0):
			newstr += t+" "
	data[i] = newstr

#some final cleaning - remove single letters
for i in range(0, len(data), 1):
        tok = tokenize_only(data[i])
        newstr = ""
        for t in tok:
                if(len(t) != 1):
                        newstr += t+" "
        data[i] = newstr
'''

#create bag of words feature vectors (tfid)
#vec = CountVectorizer(max_features=10000)
#vec = CountVectorizer(tokenizer=tokenize_and_stem, analyzer='word', max_features=10000, stop_words='english')

#vec = CountVectorizer(tokenizer=tokenize_and_stem, max_features=10000,min_df=5,max_df=.5)

#hasher = HashingVectorizer(n_features=10000, stop_words='english', non_negative=True, norm=None, binary=False)
#vec = make_pipeline(hasher, TfidfTransformer())

#vec = TfidfVectorizer(max_features=100, stop_words='english', use_idf=True, tokenizer=tokenize_and_stem, analyzer='word', min_df=5,max_df=.5)
#vec = TfidfVectorizer(max_features=10000, stop_words='english', use_idf=True, tokenizer=tokenize_only, analyzer='word', min_df=2, max_df=.2)
#vec = TfidfVectorizer(max_features=10000, stop_words='english', use_idf=True, tokenizer=tokenize_and_stem, analyzer='word')
#vec = TfidfVectorizer(max_features=10000, stop_words='english', use_idf=True)
#vec = TfidfVectorizer(max_features=10000, use_idf=True)
vec = TfidfVectorizer(max_features=10000, stop_words='english', use_idf=True, tokenizer=tokenize_and_stem, analyzer='word', min_df=2, max_df=0.5)
#vec = TfidfVectorizer(max_features=10000, stop_words='english', use_idf=True, min_df=20, max_df=0.5)

#create new features from data
train = []
test = []
trainlabels = []
testlabels = []
for i in range(0, int(len(data)/2), 1):
    train.append(data[i])
    trainlabels.append(labels[i])
for i in range(int(len(data)/2), len(data), 1):
    test.append(data[i])
    testlabels.append(labels[i])

vec.fit(train)
data = vec.transform(train).toarray()
#data = vec.fit_transform(tags_stemmed).toarray()

print("Feature names")
print(vec.get_feature_names())

print("Stop words")
print(vec.get_stop_words())

print("Data shape:", data.shape)

clf = svm.LinearSVC()
clf.fit(data, trainlabels)
testdata = vec.transform(test).toarray()
pred = clf.predict(testdata)
error = 0
for i in range(0, len(pred), 1):
    if(pred[i] != testlabels[i]):
        error += 1

print("Error = ", error/float(len(pred)))

#scores = cross_val_score(clf, data, labels, cv=10)
#print(scores)

#terms = vec.get_feature_names()
#for i in range(0, len(terms), 1):
#        print("i=",i, terms[i])

#print("Stop words")
#l = list(vec.stop_words_)
#for i in range(0, len(l), 1):
#        print(l[i])

##NOT doing this for now - we want to cluster original data to obtain keywords###
#svd = TruncatedSVD(100)
#normalizer = Normalizer(copy=False)
#lsa = make_pipeline(svd, normalizer)
#data_new = lsa.fit_transform(data)
#data = data_new
#explained_variance = svd.explained_variance_ratio_.sum()
#print("Explained variance of the SVD step: {}%".format(int(explained_variance * 100)))

'''
for i in range(0, len(data)):
    sys.stdout.write(str(labels[i])+" ")
#   if(labels[i] == 1):
#       sys.stdout.write("1 ")
#   else:
#       sys.stdout.write("-1 ")
    for j in range(0, len(data[i])):
        if(data[i][j] != 0):
            sys.stdout.write(str(j+1)+":"+str(data[i][j])+" ")
    print()
'''

'''
terms = vec.get_feature_names()
for i in range(0, len(terms), 1):
    print(terms[i])

print("Stop words")
l = list(vec.stop_words_)
for i in range(0, len(l), 1):
    print(l[i])
'''

## Check classification of test data ##

'''
#f = open("filtered_emails")
f = open("noemails_data.nonascii.test")
tags2=f.readlines()

origtags2 = []
for i in range(0, len(tags2), 1):
    origtags2.append(tags2[i])

#remove non-english words
words = set(nltk.corpus.words.words())
for i in range(0, len(tags2), 1):
    tags2[i] = " ".join(w for w in nltk.wordpunct_tokenize(tags2[i]) if w.lower() in words or not w.isalpha())

data2 = vec.transform(tags2).toarray()
for i in range(0, len(data2), 1):
    bestd=10000000
    bestc=0
    for j in range(0, num_clusters, 1):
        d=0
        for k in range(0, len(data2[0]), 1):
            d += (km.cluster_center_[j][k]-data2[i][k])**2
        d = sqrt(d)
    
        if(d < bestd):
            bestd=d
            bestc=j
    
    print(origtags2[i], keywords[bestc])
'''
